from hydromt_fiat.fiat import FiatModel
from hydromt.log import setuplog
from pathlib import Path
import pytest
import shutil
import pandas as pd

EXAMPLEDIR = Path(
    "P:/11207949-dhs-phaseii-floodadapt/Model-builder/Delft-FIAT/local_test_database"
)
DATADIR = Path().absolute() / "hydromt_fiat" / "data"

_cases = {
    "vulnerability": {
        "data_catalogue": DATADIR / "hydromt_fiat_catalog_USA.yml",
        "dir": "test_hazus_vulnerability_curves",
        "configuration": {
            "setup_vulnerability": {
                "vulnerability_fn": "default_vulnerability_curves",
                "vulnerability_identifiers_and_linking_fn": "default_hazus_iwr_linking",
                "functions_mean": "default",
                "functions_max": ["AGR1"],
                "unit": "feet",
                "step_size": 0.1,
            }
        },
    },
}


@pytest.mark.parametrize("case", list(_cases.keys()))
def test_hazus_vulnerability_curves(case):
    # Read model in examples folder.
    root = EXAMPLEDIR.joinpath(_cases[case]["dir"])
    if root.exists():
        shutil.rmtree(root)

    data_catalog_yml = str(_cases[case]["data_catalogue"])
    logger = setuplog("hydromt_fiat", log_level=10)

    fm = FiatModel(root=root, mode="w", data_libs=[data_catalog_yml], logger=logger)
    fm.build(opt=_cases[case]["configuration"])
    fm.write()

    # Check if the vulnerability data exists
    vulnerability_curves_path = root.joinpath(
        "vulnerability", "vulnerability_curves.csv"
    )
    assert vulnerability_curves_path.exists()

    # Check if the vulnerability data is correct by comparing it to data that is known
    # to be correct
    vulnerability_curves = pd.read_csv(vulnerability_curves_path)
    correct_vulnerability_curves = pd.read_csv(
        Path(__file__).parent
        / "data"
        / "correct"
        / "test_hazus_vulnerability_curves.csv"
    )

    assert vulnerability_curves.equals(correct_vulnerability_curves)
